# -*- coding: utf-8 -*-
import unittest
from odin.mapping import helpers


class MappingHelpersTestCase(unittest.TestCase):
    def test_sum_fields(self):
        self.assertEqual(42, helpers.sum_fields(22, 20))

    def test_cat_fields(self):
        self.assertEqual("abcd", helpers.cat_fields()("ab", "cd"))
        self.assertEqual("ab-cd", helpers.cat_fields('-')("ab", "cd"))

    def test_split_field(self):
        self.assertEqual(["ab", "cd"], helpers.split_field()("ab cd"))
        self.assertEqual(["abcd"], helpers.split_field('-')("abcd"))
        self.assertEqual(["ab", "cd"], helpers.split_field('-')("ab-cd"))
        self.assertEqual(["ab", "cd", "ef"], helpers.split_field('-')("ab-cd-ef"))
        self.assertEqual(["ab", "cd-ef"], helpers.split_field('-', 1)("ab-cd-ef"))



__authors__ = "Tim Savage"
__author_email__ = "tim@savage.company"
__copyright__ = "Copyright (C) 2014 Tim Savage"
__version__ = "0.6.0.1"

from odin.fields import *  # noqa
from odin.fields.composite import *  # noqa
from odin.fields.virtual import *  # noqa
from odin.mapping import *  # noqa
from odin.resources import Resource  # noqa
from odin.adapters import ResourceAdapter  # noqa



# -*- coding: utf-8 -*-
import unittest
from odin import decorators
from odin.codecs import json_codec
from odin.exceptions import ValidationError
from .resources import Author


class DecoratorsTestCase(unittest.TestCase):
    def test_returns_resource(self):
        @decorators.returns_resource(resource=Author)
        def test():
            return {"name": "Foo"}
        target = test()
        self.assertEqual("Foo", target.name)

    def test_returns_resource_full_clean(self):
        @decorators.returns_resource(resource=Author)
        def test_full_clean():
            return {"name": None}

        @decorators.returns_resource(resource=Author, full_clean=False)
        def test_unclean():
            return {"name": None}

        with self.assertRaises(ValidationError) as c:
            test_full_clean()

        target = test_unclean()
        self.assertIsNone(target.name)

    def test_returns_resource_with_codec(self):
        @decorators.returns_resource(resource=Author, codec=json_codec)
        def test():
            return '{"name": "Foo"}'

        target = test()
        self.assertEqual("Foo", target.name)



# -*- coding: utf-8 -*-
import collections
import six
from odin import registration
from odin.resources import Resource, ResourceIterable
from odin.fields.composite import ListOf, DictAs
from odin.exceptions import MappingSetupError, MappingExecutionError
from odin.mapping.helpers import MapListOf, MapDictAs, NoOpMapper
from odin.utils import cached_property

__all__ = ('Mapping', 'map_field', 'map_list_field', 'assign_field', 'define', 'assign')


force_tuple = lambda x: x if isinstance(x, (list, tuple)) else (x,)
EMPTY_LIST = tuple()


def define(from_field=None, action=None, to_field=None, to_list=False, bind=False, skip_if_none=False):
    """
    Helper method for defining a mapping.

    :param from_field: Source field to map from.
    :param action: Action callable to perform during mapping, accepted fields differ based on options.
    :param to_field: Destination field to map to; if not specified the from_field
    :param to_list: Assume the result is a list (rather than a multi value tuple).
    :param bind: During the mapping operation the first parameter should be the mapping instance.
    :param skip_if_none: If the from field is :const:`None` do not include the field (this allows the destination
        object to define it's own defaults etc)
    :return: A mapping definition.

    """
    if from_field is None and to_field is None:
        raise MappingSetupError("Either `from_field` or `to_field` must be defined.")
    return from_field, action, to_field or from_field, to_list, bind, skip_if_none


def assign(to_field, action, to_list=False, bind=True, skip_if_none=False):
    """
    Helper method for defining an assignment mapping.

    :param to_field: Destination field to map to; if not specified the from_field
    :param action: Action callable to perform during mapping, accepted fields differ based on options.
    :param to_list: Assume the result is a list (rather than a multi value tuple).
    :param bind: During the mapping operation the first parameter should be the mapping instance; defaults to True
    :param skip_if_none: If the from field is :const:`None` do not include the field (this allows the destination
        object to define it's own defaults etc)
    :return: A mapping definition.

    """
    if to_field is None:
        raise MappingSetupError("`to_field` must be defined.")
    return None, action, to_field, to_list, bind, skip_if_none


class FieldResolverBase(object):
    """Base class for field resolver objects"""
    def __init__(self, obj):
        self.obj = obj

    @cached_property
    def from_field_dict(self):
        """Property accessor for the attribute dict"""
        return self.get_from_field_dict()

    def get_from_field_dict(self):
        """ Return a field map of source fields consisting of an attribute and a Field object (if one is used).

        For resource objects the field object would be an Odin resource field, for Django models a model field etc. If
        you are building a generic object mapper the field object can be :const:`None`.

        The field object is used to determine certain automatic mapping operations (ie Lists of Odin resources to other
        Odin resources).

        :return: Dictionary
        """
        return self.get_field_dict()

    @cached_property
    def to_field_dict(self):
        """Property accessor for the attribute dict"""
        return self.get_to_field_dict()

    def get_to_field_dict(self):
        """ Return a field map consisting of an attribute and a Field object (if one is used).

        For resource objects the field object would be an Odin resource field, for Django models a model field etc. If
        you are building a generic object mapper the field object can be :const:`None`.

        The field object is used to determine certain automatic mapping operations (ie Lists of Odin resources to other
        Odin resources).

        :return: Dictionary
        """
        return self.get_field_dict()

    def get_field_dict(self):
        """ Return a field map consisting of an attribute and a Field object (if one is used).

        For resource objects the field object would be an Odin resource field, for Django models a model field etc. If
        you are building a generic object mapper the field object can be :const:`None`.

        The field object is used to determine certain automatic mapping operations (ie Lists of Odin resources to other
        Odin resources).

        :return: Dictionary
        """
        raise NotImplementedError()


class ResourceFieldResolver(FieldResolverBase):
    """Field resolver for Odin resource objects."""
    def get_field_dict(self):
        """Return a dictionary of fields along with their names."""
        return self.obj._meta.field_map

registration.register_field_resolver(ResourceFieldResolver, Resource)


def _generate_auto_mapping(name, from_fields, to_fields):
    """
    Generate the auto mapping between two fields.
    """
    from_field = from_fields[name]
    to_field = to_fields[name]
    name = force_tuple(name)

    # Handle ListOf fields
    if isinstance(from_field, ListOf) and isinstance(to_field, ListOf):
        try:
            mapping = registration.get_mapping(from_field.of, to_field.of)
            return define(name, MapListOf(mapping), name, to_list=False, bind=True)
        except KeyError:
            # If both items are from and to fields refer to the same object automatically use a mapper that just
            # produces a clone.
            if from_field.of is to_field.of:
                return define(name, MapListOf(NoOpMapper), name, to_list=False, bind=True)

    # Handle DictAs fields
    elif isinstance(from_field, DictAs) and isinstance(to_field, DictAs):
        try:
            mapping = registration.get_mapping(from_field.of, to_field.of)
            return define(name, MapDictAs(mapping), name, bind=True)
        except KeyError:
            # If both items are from and to fields refer to the same object automatically use a mapper that just
            # produces a clone.
            if from_field.of is to_field.of:
                return define(name, MapDictAs(NoOpMapper), name, bind=True)

    return define(name, None, name)


class MappingMeta(type):
    """
    Meta-class for all Mappings
    """
    def __new__(cls, name, bases, attrs):
        super_new = super(MappingMeta, cls).__new__

        # attrs will never be empty for classes declared in the standard way
        # (ie. with the `class` keyword). This is quite robust.
        if name == 'NewBase' and attrs == {}:
            return super_new(cls, name, bases, attrs)

        parents = [b for b in bases if isinstance(b, MappingMeta) and not (b.__name__ == 'NewBase'
                                                                           and b.__mro__ == (b, MappingBase, object))]
        if not parents:
            # If this isn't a subclass of Mapping, don't do anything special.
            return super_new(cls, name, bases, attrs)

        # Backward compatibility from_resource -> from_obj
        from_obj = attrs.setdefault('from_obj', attrs.get('from_resource'))
        if from_obj is None:
            raise MappingSetupError('`from_obj` is not defined.')
        to_obj = attrs.setdefault('to_obj', attrs.get('to_resource'))
        if to_obj is None:
            raise MappingSetupError('`to_obj` is not defined.')

        # Check if we have already created this mapping
        try:
            return registration.get_mapping(from_obj, to_obj)
        except KeyError:
            pass  # Not registered

        # Get field resolver objects
        try:
            from_fields = registration.get_field_resolver(from_obj).from_field_dict
        except KeyError:
            raise MappingSetupError('`from_obj` %r does not have an attribute resolver defined.' % from_obj)
        try:
            to_fields = registration.get_field_resolver(to_obj).to_field_dict
        except KeyError:
            raise MappingSetupError('`to_obj` %r does not have an attribute resolver defined.' % to_obj)

        def attr_mapping_to_mapping_rule(m, def_type, ref):
            """ Parse, validate and normalise defined mapping rules so rules can be executed without having to perform
            checks during a mapping operation."""
            to_list = False
            bind = False
            skip_if_none = False
            is_assignment = False
            try:
                map_from, action, map_to, to_list, bind, skip_if_none = m
            except ValueError:
                try:
                    map_from, action, map_to = m
                except ValueError:
                    raise MappingSetupError('Bad mapping definition `%s` in %s `%s`.' % (m, def_type, ref))

            if map_from is None:
                is_assignment = True

            if not is_assignment:
                map_from = force_tuple(map_from)
                for f in map_from:
                    if f not in from_fields:
                        raise MappingSetupError('Field `%s` of %s `%s` not found on from object. ' % (f, def_type, ref))

            if isinstance(action, six.string_types):
                if action not in attrs:
                    raise MappingSetupError('Action named %s defined in %s `%s` was not defined on mapping object.' % (
                        action, def_type, ref))
                if not callable(attrs[action]):
                    raise MappingSetupError('Action named %s defined in %s `%s` is not callable.' % (
                        action, def_type, ref))
            elif action is not None and not callable(action):
                raise MappingSetupError('Action on %s `%s` is not callable.' % (def_type, ref))
            elif action is None and is_assignment:
                raise MappingSetupError('No action supplied for `%s` in `%s`.' % (def_type, ref))

            map_to = force_tuple(map_to)
            if to_list and len(map_to) != 1:
                raise MappingSetupError('The %s `%s` specifies a to_list mapping, these can only be applied to a '
                                        'single target field.' % (def_type, m))
            for f in map_to:
                if f not in to_fields:
                    raise MappingSetupError('Field `%s` of %s `%s` not found on to object. ' % (f, def_type, ref))

            return map_from, action, map_to, to_list, bind, skip_if_none

        # Determine what fields need to have mappings generated
        exclude_fields = attrs.get('exclude_fields') or tuple()
        unmapped_fields = [attname for attname in from_fields if attname not in exclude_fields]

        def remove_from_unmapped_fields(rule):
            # Remove any fields that are handled by a mapping rule from unmapped_fields list.
            map_to = rule[2]
            if len(map_to) == 1 and map_to[0] in unmapped_fields:
                unmapped_fields.remove(map_to[0])

        # Generate mapping rules.
        mapping_rules = []

        # Check that from_obj is a sub_class (or same class) as any `parent.from_obj`. This is important for mapping
        # sub class lists and resolving mappings.
        base_parents = [p for p in parents if hasattr(p, '_subs')]
        for p in base_parents:
            if not issubclass(from_obj, p.from_obj):
                raise MappingSetupError('`from_obj` must be a subclass of `parent.from_obj`')
            if not issubclass(to_obj, p.to_obj):
                raise MappingSetupError('`to_obj` must be a subclass of `parent.to_obj`')

            # Copy mapping rules
            for mapping_rule in p._mapping_rules:
                mapping_rules.append(mapping_rule)
                remove_from_unmapped_fields(mapping_rule)

        # Add basic mappings
        for idx, mapping in enumerate(attrs.pop('mappings', [])):
            mapping_rule = attr_mapping_to_mapping_rule(mapping, 'basic mapping', idx)
            mapping_rules.append(mapping_rule)
            remove_from_unmapped_fields(mapping_rule)

        # Add custom mappings
        for attr in attrs.values():
            # Methods with a _mapping attribute have been decorated by either `map_field` or `map_list_field`
            # decorators.
            if hasattr(attr, '_mapping'):
                mapping_rule = attr_mapping_to_mapping_rule(getattr(attr, '_mapping'), 'custom mapping', attr)
                mapping_rules.append(mapping_rule)
                remove_from_unmapped_fields(mapping_rule)
                # Remove mapping
                delattr(attr, '_mapping')

        # Add auto mapped fields that are yet to be mapped.
        for field in unmapped_fields:
            if field in to_fields:
                mapping_rules.append(_generate_auto_mapping(field, from_fields, to_fields))

        # Update attributes
        attrs['_mapping_rules'] = mapping_rules
        attrs['_subs'] = {}

        registration.register_mapping(super_new(cls, name, bases, attrs))
        mapper = registration.get_mapping(from_obj, to_obj)

        # Register mapping with parents mapping objects as a sub class.
        for parent in base_parents:
            parent._subs[from_obj] = mapper

        return mapper


class MappingResult(ResourceIterable):
    """
    Iterator used to return a sequence from a mapping operation (used by ``Mapping.apply``).
    """
    def __init__(self, sequence, mapping, context=None, *mapping_options):
        super(MappingResult, self).__init__(sequence)
        self.mapping = mapping
        self.context = context or {}
        self.context.setdefault('_loop_idx', [])
        self.mapping_options = mapping_options

    def __iter__(self):
        self.context['_loop_idx'].append(0)
        for item in self.sequence:
            yield self.mapping.apply(item, self.context, *self.mapping_options)
            self.context['_loop_idx'][-1] += 1
        self.context['_loop_idx'].pop()


class MappingBase(object):
    from_obj = None
    to_obj = None

    # Pending deprecation, move to from_obj and to_obj terminology
    from_resource = None
    to_resource = None

    @classmethod
    def apply(cls, source_obj, context=None, allow_subclass=False):
        """
        Apply conversion either a single resource or a list of resources using the mapping defined by this class.

        If a list of resources is supplied an iterable is returned.

        :param source_obj: The source resource, this must be an instance of :py:attr:`Mapping.from_obj`.
        :param context: An optional context value, this can be any value you want to aid in mapping
        """
        context = context or {}
        context.setdefault('_loop_idx', [])

        if isinstance(source_obj, (list, tuple)) or hasattr(source_obj, '__iter__'):
            return MappingResult(source_obj, cls, context, allow_subclass)
        elif source_obj.__class__ is cls.from_obj:
            return cls(source_obj, context).convert()
        else:
            # Sub class lookup required
            sub_mapping = cls._subs.get(source_obj.__class__)
            if sub_mapping:
                return sub_mapping(source_obj, context).convert()
            if allow_subclass:
                if allow_subclass and isinstance(source_obj, cls.from_obj):
                    return cls(source_obj, context, True).convert()

                raise TypeError('`source_resource` parameter must be an instance (or subclass instance) of %s' %
                                cls.from_obj)

            raise TypeError('`source_resource` parameter must be an instance of %s' % cls.from_obj)

    def __init__(self, source_obj, context=None, allow_subclass=False):
        """
        Initialise instance of mapping.

        :param source_obj: The source resource, this must be an instance of :py:attr:`Mapping.from_obj`.
        :param context: An optional context value, this can be any value you want to aid in mapping
        """
        if allow_subclass:
            if not isinstance(source_obj, self.from_obj):
                raise TypeError('`source_resource` parameter must be an instance of subclass of %s' % self.from_obj)
        else:
            if source_obj.__class__ is not self.from_obj:
                raise TypeError('`source_resource` parameter must be an instance of %s' % self.from_obj)
        self.source = source_obj
        self.context = context or {}

    @property
    def loop_idx(self):
        """
        Index within a loop of this mapping (note loop might be for a parent object)
        :returns: Index within the loop; or `None` if we are not currently in a loop.
        """
        loops = self.context.setdefault('_loop_idx', [])
        return loops[-1] if loops else None

    @property
    def loop_level(self):
        """
        How many layers of loops are we in?
        """
        return len(self.context.setdefault('_loop_idx', []))

    @property
    def in_loop(self):
        """
        Is this mapping currently in a loop?
        """
        return bool(self.context.setdefault('_loop_idx', []))

    def _apply_rule(self, mapping_rule):
        # Unpack mapping definition and fetch from values
        from_fields, action, to_fields, to_list, bind, skip_if_none = mapping_rule

        # This is an assignment rather than a mapping
        if from_fields is None:
            from_values = EMPTY_LIST
        else:
            from_values = tuple(getattr(self.source, f) for f in from_fields)

        if action is None:
            to_values = from_values
        else:
            if isinstance(action, six.string_types):
                action = getattr(self, action)

            try:
                if bind:
                    to_values = action(self, *from_values)
                else:
                    to_values = action(*from_values)
            except TypeError as ex:
                raise MappingExecutionError('%s applying rule %s' % (ex, mapping_rule))

        if to_list:
            if isinstance(to_values, collections.Iterable):
                to_values = (list(to_values),)
        else:
            to_values = force_tuple(to_values)

        if len(to_fields) != len(to_values):
            raise MappingExecutionError('Rule expects %s fields (%s returned) applying rule %s' % (
                len(to_fields), len(to_values), mapping_rule))

        if skip_if_none:
            return {f: to_values[i] for i, f in enumerate(to_fields) if to_values[i] is not None}
        else:
            return {f: to_values[i] for i, f in enumerate(to_fields)}

    def create_object(self, **field_values):
        """
        Create an instance of target object, this method can be customise to handle custom object initialisation.

        :param field_values: Dictionary of values for creating the target object.

        """
        return self.to_obj(**field_values)

    def convert(self, **field_values):
        """
        Convert the provided source into a destination object.

        :param field_values: Initial field values (or fields not provided by source object);

        """
        assert hasattr(self, '_mapping_rules')

        values = field_values

        for mapping_rule in self._mapping_rules:
            values.update(self._apply_rule(mapping_rule))

        return self.create_object(**values)

    def update(self, destination_obj, ignore_fields=None, fields=None):
        """
        Update an existing object with fields from the provided source object.

        :param destination_obj: The existing destination object.
        :param ignore_fields: A list of fields that should be ignored eg ID fields

        """
        assert hasattr(self, '_mapping_rules')

        ignore_fields = ignore_fields or []

        for mapping_rule in self._mapping_rules:
            for name, value in self._apply_rule(mapping_rule).items():
                if name in ignore_fields or (fields and name not in fields):
                    continue
                setattr(destination_obj, name, value)

        return destination_obj

    def diff(self, destination_obj):
        """
        Return all fields that are different.

        :note: a full mapping operation is performed during the diffing process.

        :param destination_obj: The existing destination object.
        :return: set of fields that vary.

        """
        assert hasattr(self, '_mapping_rules')

        diff_fields = set()
        for mapping_rule in self._mapping_rules:
            for name, value in self._apply_rule(mapping_rule).items():
                if value != getattr(destination_obj, name):
                    diff_fields.add(name)
        return diff_fields


class Mapping(six.with_metaclass(MappingMeta, MappingBase)):
    """
    Definition of a mapping between two Objects.
    """
    exclude_fields = []
    mappings = []


def map_field(func=None, from_field=None, to_field=None, to_list=False):
    """
    Field decorator for custom mappings.

    :param from_field: Name of the field to map from; default is to use the function name.
    :param to_field: Name of the field to map to; default is to use the function name.
    :param to_list: The result is a list (rather than a multi value tuple).
    """
    def inner(func):
        func._mapping = define(
            from_field or func.__name__,
            func.__name__,
            to_field or func.__name__,
            to_list
        )
        return func

    return inner(func) if func else inner


def map_list_field(*args, **kwargs):
    """
    Field decorator for custom mappings that return a single list.

    This mapper also allows for returning an iterator or generator that will be converted into a list during the
    mapping operation.

    Parameters are identical to the :py:meth:`map_field` method except ``to_list`` which is forced to be True.
    """
    kwargs['to_list'] = True
    return map_field(*args, **kwargs)


def assign_field(func=None, to_field=None, to_list=False):
    """
    Field decorator for assigning a value to destination field without requiring a corresponding source field.

    Allows for the mapping to calculate a value based on the context or other information. Useful when a destination
    objects defaulting mechanism is not able to calculate a default that either applies or is suitable.

    :param to_field: Name of the field to assign value to; default is to use the function name.
    :param to_list: The result is a list (rather than a multi value tuple).
    """
    def inner(func):
        func._mapping = define(
            None,
            func.__name__,
            to_field or func.__name__,
            to_list
        )
        return func

    return inner(func) if func else inner


def mapping_factory(from_obj, to_obj, base_mapping=Mapping, generate_reverse=True,
                    mappings=None, reverse_mappings=None,
                    exclude_fields=None, reverse_exclude_fields=None):
    """
    Factory method for generating simple mappings between objects.

    A common use-case for this method is in generating mappings in baldr's ``model_resource_factory`` method that
    auto-generates resources from Django models.

    :param from_obj: Object to map from.
    :param to_obj: Object to map to.
    :param base_mapping: Base mapping class; default is ``odin.Mapping``.
    :param generate_reverse: Generate the reverse of the mapping ie swap from_obj and to_obj.
    :param mappings: User provided mappings (this is equivalent ot ``odin.Mapping.mappings``)
    :param reverse_mappings: User provided reverse mappings (this is equivalent ot ``odin.Mapping.mappings``). Only
        used if ``generate_reverse`` is True.
    :param exclude_fields: Fields to exclude from auto-generated mappings
    :param reverse_exclude_fields: Fields to exclude from auto-generated reverse mappings.
    :return: if generate_reverse is True a tuple(forward_mapping, reverse_mapping); else just the forward_mapping.

    """
    forward_mapping = type(
        "%s%sTo%s%s" % (
            from_obj.__class__.__name__, from_obj.__name__,
            to_obj.__class__.__name__, to_obj.__name__
        ),
        (base_mapping, ),
        dict(
            from_obj=from_obj,
            to_obj=to_obj,
            exclude_fields=exclude_fields or list(),
            mappings=mappings or dict()
        )
    )

    if generate_reverse:
        reverse_mapping = type(
            "%s%sTo%s%s" % (
                to_obj.__class__.__name__, to_obj.__name__,
                from_obj.__class__.__name__, from_obj.__name__,
            ),
            (base_mapping, ),
            dict(
                from_obj=to_obj,
                to_obj=from_obj,
                exclude_fields=reverse_exclude_fields or list(),
                mappings=reverse_mappings or dict()
            )
        )

        return forward_mapping, reverse_mapping

    return forward_mapping


def forward_mapping_factory(from_obj, to_obj, base_mapping=Mapping, mappings=None, exclude_fields=None):
    """
    Factory method for generating simple forward mappings between objects.

    :param from_obj: Object to map from.
    :param to_obj: Object to map to.
    :param base_mapping: Base mapping class; default is ``odin.Mapping``.
    :param mappings: User provided mappings (this is equivalent ot ``odin.Mapping.mappings``)
    :param exclude_fields: Fields to exclude from auto-generated mappings
    :return: Forward_mapping object.

    """
    return mapping_factory(from_obj, to_obj, base_mapping, False, mappings, None, exclude_fields, None)



# -*- coding: utf-8 -*-
from copy import deepcopy
import unittest
import datetime
from odin.fields import *
from odin.fields import Field, TimeStampField
from odin.datetimeutil import utc, FixedTimezone
from odin.fields.virtual import VirtualField
from odin.validators import MinValueValidator, MaxValueValidator, MaxLengthValidator, RegexValidator
from odin.exceptions import ValidationError


class ObjectValue(object):
    pass


class TestValidator(object):
    message = 'Default message'
    code = 'test_code'

    def __init__(self, fail, *params):
        self.fail = fail
        self.params = params

    def __call__(self, value):
        if self.fail:
            raise ValidationError(code=self.code, message=self.message, params=self.params)


class FieldTestCase(unittest.TestCase):
    def test_error_messages_no_overrides(self):
        target = Field()

        self.assertDictEqual({
            'invalid_choice': 'Value %r is not a valid choice.',
            'null': 'This field cannot be null.',
            'required': 'This field is required.',
        }, target.error_messages)

    def test_error_messages_override_add(self):
        target = Field(error_messages={
            'null': 'Override',
            'other': 'Other Value',
        })

        self.assertDictEqual({
            'invalid_choice': 'Value %r is not a valid choice.',
            'null': 'Override',
            'required': 'This field is required.',
            'other': 'Other Value',
        }, target.error_messages)

    def test_set_attributes_from_name(self):
        target = Field()
        target.set_attributes_from_name("test_name")

        self.assertEqual("test_name", target.name)
        self.assertEqual("test_name", target.attname)
        self.assertEqual("test name", target.verbose_name)
        self.assertEqual("test names", target.verbose_name_plural)

    def test_set_attributes_from_name_with_name(self):
        target = Field(name="init_name")
        target.set_attributes_from_name("test_name")

        self.assertEqual("init_name", target.name)
        self.assertEqual("test_name", target.attname)
        self.assertEqual("init name", target.verbose_name)
        self.assertEqual("init names", target.verbose_name_plural)

    def test_set_attributes_from_name_with_verbose_name(self):
        target = Field(verbose_name="init Verbose Name")
        target.set_attributes_from_name("test_name")

        self.assertEqual("test_name", target.name)
        self.assertEqual("test_name", target.attname)
        self.assertEqual("init Verbose Name", target.verbose_name)
        self.assertEqual("init Verbose Names", target.verbose_name_plural)

    def test_has_default(self):
        target = Field()

        self.assertFalse(target.has_default())

    def test_has_default_supplied(self):
        target = Field(default="123")

        self.assertTrue(target.has_default())

    def test_get_default(self):
        target = Field()

        self.assertIsNone(target.get_default())

    def test_get_default_supplied(self):
        target = Field(default="123")

        self.assertEqual("123", target.get_default())

    def test_get_default_callable(self):
        target = Field(default=lambda: "321")

        self.assertEqual("321", target.get_default())

    def test_value_from_object(self):
        target = Field()
        target.set_attributes_from_name("test_name")

        an_obj = ObjectValue()
        setattr(an_obj, "test_name", "test_value")

        actual = target.value_from_object(an_obj)
        self.assertEqual("test_value", actual)

    def test__repr(self):
        target = Field()
        self.assertEqual("<odin.fields.Field>", repr(target))
        target.set_attributes_from_name("eek")
        self.assertEqual("<odin.fields.Field: eek>", repr(target))

    def test__deep_copy(self):
        field = Field(name="Test")
        target_copy = deepcopy(field)
        target_assign = field
        self.assertIs(field, target_assign)
        self.assertIsNot(field, target_copy)
        self.assertEqual(field.name, target_copy.name)

    def test_run_validators_and_override_validator_message(self):
        target = Field(error_messages={'test_code': 'Override message'}, validators=[TestValidator(True)])

        try:
            target.run_validators("Placeholder")
        except ValidationError as ve:
            self.assertEqual('Override message', ve.messages[0])
        else:
            raise AssertionError("Validation Error not raised.")

    def test_run_validators_and_override_validator_message_with_params(self):
        target = Field(error_messages={'test_code': 'Override message: %s'}, validators=[TestValidator(True, "123")])

        try:
            target.run_validators("Placeholder")
        except ValidationError as ve:
            self.assertEqual(['Override message: 123'], ve.messages)
        else:
            raise AssertionError("Validation Error not raised.")

    def test_default_to_python_raises_not_implemented(self):
        target = Field()
        self.assertRaises(NotImplementedError, target.to_python, "Anything...")


class TestVirtualField(VirtualField):
    pass


class VirtualFieldTestCase(unittest.TestCase):
    def test_creation_counter(self):
        current_count = VirtualField.creation_counter
        next_count = current_count + 1
        target = VirtualField()

        assert VirtualField.creation_counter == next_count
        assert target.creation_counter == current_count
        assert hash(target) == current_count

    def test_repr(self):
        target = TestVirtualField()
        self.assertEqual("<tests.test_fields.TestVirtualField>", repr(target))
        target.set_attributes_from_name("eek")
        self.assertEqual("<tests.test_fields.TestVirtualField: eek>", repr(target))

    def test_default_descriptor_behaviour(self):
        class TestObj(object):
            test_field = VirtualField()
        target = TestObj()

        with self.assertRaises(NotImplementedError):
            _ = target.test_field

        with self.assertRaises(AttributeError) as cm:
            target.test_field = 123

        self.assertEqual("Read only", str(cm.exception))


class FieldsTests(unittest.TestCase):
    def assertValidatorIn(self, validatorClass, validators):
        """
        Assert that the specified validator is in the validation list.
        :param validatorClass:
        :param validators:
        """
        for v in validators:
            if isinstance(v, validatorClass):
                return
        raise AssertionError("Validator %r was not found in list of validators." % validatorClass)

    def assertValidatorNotIn(self, validatorClass, validators):
        """
        Assert that the specified validator is not in the validation list.
        :param validatorClass:
        :param validators:
        """
        for v in validators:
            if isinstance(v, validatorClass):
                raise AssertionError("Validator %r was found in list of validators." % validatorClass)

    # BooleanField ############################################################

    def test_booleanfield_1(self):
        f = BooleanField()
        self.assertRaises(ValidationError, f.clean, None)
        self.assertRaises(ValidationError, f.clean, '')
        self.assertEqual(True, f.clean(True))
        self.assertEqual(True, f.clean(1))
        self.assertEqual(True, f.clean('Yes'))
        self.assertEqual(True, f.clean('true'))
        self.assertEqual(True, f.clean('T'))
        self.assertEqual(True, f.clean('1'))
        self.assertRaises(ValidationError, f.clean, 'Awesome!')
        self.assertEqual(False, f.clean(False))
        self.assertEqual(False, f.clean(0))
        self.assertEqual(False, f.clean('No'))
        self.assertEqual(False, f.clean('false'))
        self.assertEqual(False, f.clean('F'))
        self.assertEqual(False, f.clean('0'))

    # StringField #############################################################

    def test_stringfield_1(self):
        f = StringField()
        self.assertEqual('1', f.clean('1'))
        self.assertEqual('eek', f.clean('eek'))
        self.assertRaises(ValidationError, f.clean, None)
        self.assertEqual('[1, 2, 3]', f.clean([1, 2, 3]))
        self.assertEqual(f.max_length, None)
        self.assertValidatorNotIn(MaxLengthValidator, f.validators)

    def test_stringfield_2(self):
        f = StringField(null=True)
        self.assertEqual('1', f.clean('1'))
        self.assertEqual('eek', f.clean('eek'))
        self.assertEqual(None, f.clean(None))
        self.assertEqual('[1, 2, 3]', f.clean([1, 2, 3]))
        self.assertEqual(f.max_length, None)
        self.assertValidatorNotIn(MaxLengthValidator, f.validators)

    def test_stringfield_3(self):
        f = StringField(null=True, max_length=10)
        self.assertEqual(None, f.clean(None))
        self.assertEqual('', f.clean(''))
        self.assertEqual('12345', f.clean('12345'))
        self.assertEqual('1234567890', f.clean('1234567890'))
        self.assertRaises(ValidationError, f.clean, '1234567890a')
        self.assertEqual(f.max_length, 10)
        self.assertValidatorIn(MaxLengthValidator, f.validators)

    # URLField ################################################################

    def test_urlfield_1(self):
        f = UrlField()
        self.assertEqual('http://www.github.com', f.clean('http://www.github.com'))
        self.assertRaises(ValidationError, f.clean, 'eek')
        self.assertRaises(ValidationError, f.clean, None)
        self.assertEqual(f.max_length, None)
        self.assertValidatorIn(RegexValidator, f.validators)

    # IntegerField ############################################################

    def test_integerfield_1(self):
        f = IntegerField()
        self.assertRaises(ValidationError, f.clean, None)
        self.assertRaises(ValidationError, f.clean, 'abc')
        self.assertEqual(123, f.clean(123))
        self.assertEqual(123, f.clean('123'))
        self.assertEqual(123, f.clean(123.5))
        self.assertEqual(None, f.min_value)
        self.assertValidatorNotIn(MinValueValidator, f.validators)
        self.assertEqual(None, f.max_value)
        self.assertValidatorNotIn(MaxValueValidator, f.validators)

    def test_integerfield_2(self):
        f = IntegerField(null=True)
        self.assertEqual(None, f.clean(None))
        self.assertRaises(ValidationError, f.clean, 'abc')
        self.assertEqual(69, f.clean(69))
        self.assertEqual(69, f.clean('69'))
        self.assertEqual(69, f.clean(69.5))
        self.assertEqual(None, f.min_value)
        self.assertValidatorNotIn(MinValueValidator, f.validators)
        self.assertEqual(None, f.max_value)
        self.assertValidatorNotIn(MaxValueValidator, f.validators)

    def test_integerfield_3(self):
        f = IntegerField(min_value=50, max_value=100)
        self.assertRaises(ValidationError, f.clean, None)
        self.assertRaises(ValidationError, f.clean, 'abc')
        self.assertEqual(69, f.clean(69))
        self.assertEqual(69, f.clean('69'))
        self.assertEqual(69, f.clean(69.5))
        self.assertEqual(50, f.clean(50))
        self.assertEqual(100, f.clean(100))
        self.assertRaises(ValidationError, f.clean, 30)
        self.assertRaises(ValidationError, f.clean, 110)
        self.assertEqual(50, f.min_value)
        self.assertValidatorIn(MinValueValidator, f.validators)
        self.assertEqual(100, f.max_value)
        self.assertValidatorIn(MaxValueValidator, f.validators)

    # FloatField ##############################################################

    def test_floatfield_1(self):
        f = FloatField()
        self.assertRaises(ValidationError, f.clean, None)
        self.assertRaises(ValidationError, f.clean, 'abc')
        self.assertEqual(123, f.clean(123))
        self.assertEqual(123.5, f.clean('123.5'))
        self.assertEqual(123.5, f.clean(123.5))
        self.assertEqual(None, f.min_value)
        self.assertValidatorNotIn(MinValueValidator, f.validators)
        self.assertEqual(None, f.max_value)
        self.assertValidatorNotIn(MaxValueValidator, f.validators)

    def test_floatfield_2(self):
        f = FloatField(null=True)
        self.assertEqual(None, f.clean(None))
        self.assertRaises(ValidationError, f.clean, 'abc')
        self.assertEqual(69, f.clean(69))
        self.assertEqual(69.5, f.clean('69.5'))
        self.assertEqual(69.5, f.clean(69.5))
        self.assertEqual(None, f.min_value)
        self.assertValidatorNotIn(MinValueValidator, f.validators)
        self.assertEqual(None, f.max_value)
        self.assertValidatorNotIn(MaxValueValidator, f.validators)

    def test_floatfield_3(self):
        f = FloatField(min_value=50.5, max_value=100.4)
        self.assertRaises(ValidationError, f.clean, None)
        self.assertRaises(ValidationError, f.clean, 'abc')
        self.assertEqual(69, f.clean(69))
        self.assertEqual(69.5, f.clean('69.5'))
        self.assertEqual(69.5, f.clean(69.5))
        self.assertEqual(50.5, f.clean(50.5))
        self.assertEqual(100.4, f.clean(100.4))
        self.assertRaises(ValidationError, f.clean, 30)
        self.assertRaises(ValidationError, f.clean, 110)
        self.assertEqual(50.5, f.min_value)
        self.assertValidatorIn(MinValueValidator, f.validators)
        self.assertEqual(100.4, f.max_value)
        self.assertValidatorIn(MaxValueValidator, f.validators)

    # DateField ###############################################################

    def test_datefield_1(self):
        f = DateField()
        self.assertRaises(ValidationError, f.clean, None)
        self.assertRaises(ValidationError, f.clean, 'abc')
        self.assertRaises(ValidationError, f.clean, 123)
        self.assertEqual(datetime.date(2013, 11, 24), f.clean('2013-11-24'))
        self.assertEqual(datetime.date(2013, 11, 24), f.clean(datetime.date(2013, 11, 24)))
        self.assertEqual(datetime.date(2013, 11, 24), f.clean(datetime.datetime(2013, 11, 24, 1, 14)))

    def test_datefield_2(self):
        f = DateField(null=True)
        self.assertEqual(None, f.clean(None))
        self.assertRaises(ValidationError, f.clean, 'abc')
        self.assertRaises(ValidationError, f.clean, 123)
        self.assertEqual(datetime.date(2013, 11, 24), f.clean('2013-11-24'))
        self.assertEqual(datetime.date(2013, 11, 24), f.clean(datetime.date(2013, 11, 24)))
        self.assertEqual(datetime.date(2013, 11, 24), f.clean(datetime.datetime(2013, 11, 24, 1, 14)))

    # TimeField ###############################################################

    def test_timefield_1(self):
        f = TimeField(assume_local=False)
        self.assertRaises(ValidationError, f.clean, None)
        self.assertRaises(ValidationError, f.clean, 'abc')
        self.assertRaises(ValidationError, f.clean, 123)
        self.assertEqual(datetime.time(18, 43, tzinfo=utc), f.clean('18:43:00.000Z'))
        self.assertEqual(datetime.time(18, 43, tzinfo=utc), f.clean(datetime.time(18, 43, tzinfo=utc)))

    def test_timefield_2(self):
        f = TimeField(assume_local=False, null=True)
        self.assertEqual(None, f.clean(None))
        self.assertRaises(ValidationError, f.clean, 'abc')
        self.assertRaises(ValidationError, f.clean, 123)
        self.assertEqual(datetime.time(18, 43, tzinfo=utc), f.clean('18:43:00.000Z'))
        self.assertEqual(datetime.time(18, 43, tzinfo=utc), f.clean(datetime.time(18, 43, tzinfo=utc)))

    # DateTimeField ###########################################################

    def test_datetimefield_1(self):
        f = DateTimeField(assume_local=False)
        self.assertRaises(ValidationError, f.clean, None)
        self.assertRaises(ValidationError, f.clean, 'abc')
        self.assertRaises(ValidationError, f.clean, 123)
        self.assertEqual(datetime.datetime(2013, 11, 24, 18, 43, tzinfo=utc), f.clean('2013-11-24T18:43:00.000Z'))
        self.assertEqual(datetime.datetime(2013, 11, 24, 18, 43, tzinfo=utc), f.clean('2013-11-24 18:43:00.000Z'))
        self.assertEqual(datetime.datetime(2013, 11, 24, 18, 43, tzinfo=utc),
                         f.clean(datetime.datetime(2013, 11, 24, 18, 43, tzinfo=utc)))

    def test_datetimefield_2(self):
        f = DateTimeField(assume_local=False, null=True)
        self.assertEqual(None, f.clean(None))
        self.assertRaises(ValidationError, f.clean, 'abc')
        self.assertRaises(ValidationError, f.clean, 123)
        self.assertEqual(datetime.datetime(2013, 11, 24, 18, 43, tzinfo=utc), f.clean('2013-11-24T18:43:00.000Z'))
        self.assertEqual(datetime.datetime(2013, 11, 24, 18, 43, tzinfo=utc), f.clean('2013-11-24 18:43:00.000Z'))
        self.assertEqual(datetime.datetime(2013, 11, 24, 18, 43, tzinfo=utc),
                         f.clean(datetime.datetime(2013, 11, 24, 18, 43, tzinfo=utc)))

    # HttpDateTimeField #######################################################

    def test_httpdatetimefield_1(self):
        f = HttpDateTimeField()
        self.assertRaises(ValidationError, f.clean, None)
        self.assertRaises(ValidationError, f.clean, 'abc')
        self.assertRaises(ValidationError, f.clean, 123)
        self.assertEqual(datetime.datetime(2012, 8, 29, 17, 12, 58, tzinfo=utc),
                         f.clean('Wed Aug 29 17:12:58 +0000 2012'))
        self.assertEqual(datetime.datetime(2013, 11, 24, 18, 43, tzinfo=utc),
                         f.clean(datetime.datetime(2013, 11, 24, 18, 43, tzinfo=utc)))

    def test_httpdatetimefield_2(self):
        f = HttpDateTimeField(null=True)
        self.assertEqual(None, f.clean(None))
        self.assertRaises(ValidationError, f.clean, 'abc')
        self.assertRaises(ValidationError, f.clean, 123)
        self.assertEqual(datetime.datetime(2012, 8, 29, 17, 12, 58, tzinfo=utc),
                         f.clean('Wed Aug 29 17:12:58 +0000 2012'))
        self.assertEqual(datetime.datetime(2013, 11, 24, 18, 43, tzinfo=utc),
                         f.clean(datetime.datetime(2013, 11, 24, 18, 43, tzinfo=utc)))

    # TimeStampField ##########################################################

    def test_timestampfield_1(self):
        f = TimeStampField()
        self.assertRaises(ValidationError, f.clean, None)
        self.assertRaises(ValidationError, f.clean, 'abc')
        self.assertRaises(ValidationError, f.clean, 'Wed Aug 29 17:12:58 +0000 2012')
        self.assertEqual(datetime.datetime(1970, 1, 1, 0, 2, 3, tzinfo=utc), f.clean(123))
        self.assertEqual(datetime.datetime(2013, 11, 24, 18, 43, tzinfo=utc),
                         f.clean(datetime.datetime(2013, 11, 24, 18, 43, tzinfo=utc)))

    def test_timestampfield_2(self):
        f = TimeStampField(null=True)
        self.assertEqual(None, f.clean(None))
        self.assertRaises(ValidationError, f.clean, 'abc')
        self.assertRaises(ValidationError, f.clean, 'Wed Aug 29 17:12:58 +0000 2012')
        self.assertEqual(datetime.datetime(1970, 1, 1, 0, 2, 3, tzinfo=utc), f.clean(123))
        self.assertEqual(datetime.datetime(2013, 11, 24, 18, 43, tzinfo=utc),
                         f.clean(datetime.datetime(2013, 11, 24, 18, 43, tzinfo=utc)))

    def test_timestampfield_3(self):
        f = TimeStampField()
        self.assertEqual(None, f.prepare(None))
        self.assertEqual(123, f.prepare(datetime.datetime(1970, 1, 1, 0, 2, 3, tzinfo=utc)))
        self.assertEqual(123, f.prepare(
            datetime.datetime(1970, 1, 1, 10, 2, 3, tzinfo=FixedTimezone.from_hours_minutes(10))))

    # DictField ###############################################################

    def test_dictfield_1(self):
        f = DictField()
        self.assertRaises(ValidationError, f.clean, None)
        self.assertRaises(ValidationError, f.clean, 'abc')
        self.assertRaises(ValidationError, f.clean, 123)
        self.assertEqual({}, f.clean({}))
        self.assertEqual({'foo': 'bar'}, f.clean({'foo': 'bar'}))
        self.assertEqual(f.default, dict)

    def test_dictfield_2(self):
        f = DictField(null=True)
        self.assertEqual(None, f.clean(None))
        self.assertEqual({}, f.clean({}))
        self.assertRaises(ValidationError, f.clean, 'abc')
        self.assertRaises(ValidationError, f.clean, 123)
        self.assertEqual({'foo': 'bar'}, f.clean({'foo': 'bar'}))

    # ArrayField ##############################################################

    def test_arrayfield_1(self):
        f = ArrayField()
        self.assertRaises(ValidationError, f.clean, None)
        self.assertRaises(ValidationError, f.clean, 'abc')
        self.assertRaises(ValidationError, f.clean, 123)
        self.assertEqual([], f.clean([]))
        self.assertEqual(['foo', 'bar'], f.clean(['foo', 'bar']))
        self.assertEqual(['foo', 'bar', '$', 'eek'], f.clean(['foo', 'bar', '$', 'eek']))
        self.assertEqual(f.default, list)

    def test_arrayfield_2(self):
        f = ArrayField(null=True)
        self.assertEqual(None, f.clean(None))
        self.assertRaises(ValidationError, f.clean, 'abc')
        self.assertRaises(ValidationError, f.clean, 123)
        self.assertEqual([], f.clean([]))
        self.assertEqual(['foo', 'bar'], f.clean(['foo', 'bar']))
        self.assertEqual(['foo', 'bar', '$', 'eek'], f.clean(['foo', 'bar', '$', 'eek']))

    # TypedArrayField #########################################################

    def test_typedarrayfield_1(self):
        f = TypedArrayField(IntegerField())
        self.assertRaises(ValidationError, f.clean, None)
        self.assertRaises(ValidationError, f.clean, 'abc')
        self.assertRaises(ValidationError, f.clean, 123)
        self.assertEqual([], f.clean([]))
        self.assertRaises(ValidationError, f.clean, ['foo', 'bar'])
        self.assertEqual([1, 2, 3], f.clean([1, 2, 3]))
        self.assertEqual(f.default, list)

    def test_typedarrayfield_2(self):
        f = TypedArrayField(IntegerField(), null=True)
        self.assertEqual(None, f.clean(None))
        self.assertRaises(ValidationError, f.clean, 'abc')
        self.assertRaises(ValidationError, f.clean, 123)
        self.assertEqual([], f.clean([]))
        self.assertRaises(ValidationError, f.clean, ['foo', 'bar'])
        self.assertEqual([1, 2, 3], f.clean([1, 2, 3]))



# -*- coding: utf-8 -*-
import unittest
from odin import utils
from .resources import Book


class CamelAndDashFormatsTestCase(unittest.TestCase):
    def test_camel_to_lower_separated(self):
        self.assertEqual("this_is_an_example", utils.camel_to_lower_underscore('thisIsAnExample'))
        self.assertEqual("this_is_an_example", utils.camel_to_lower_underscore('ThisIsAnExample'))
        self.assertEqual("this-is-a-test", utils.camel_to_lower_dash('thisIsATest'))

    def test_lower_underscore_to_camel(self):
        self.assertEqual("thisIsAnExample", utils.lower_underscore_to_camel('this_is_an_example'))
        self.assertEqual("thisIsAnExample", utils.lower_underscore_to_camel('this_is_an_example'))
        self.assertEqual("thisIsATest", utils.lower_underscore_to_camel('this_Is_A_test'))

    def test_lower_dash_to_camel(self):
        self.assertEqual("thisIsAnExample", utils.lower_dash_to_camel('this-is-an-example'))
        self.assertEqual("thisIsAnExample", utils.lower_dash_to_camel('this-is-an-example'))
        self.assertEqual("thisIsATest", utils.lower_dash_to_camel('this-Is-A-test'))


class IteratorTestCases(unittest.TestCase):
    def test_extract_fields_from_dict(self):
        source_dict = {
            'title': "My Title",
            'num_pages': 1,
            'rrp': 1.2,
            'fiction': True,
            'published': ['no_validation_for_type'],
            'another_field': 'that is not on the resource'
        }

        self.assertDictEqual({
            'title': "My Title",
            'num_pages': 1,
            'rrp': 1.2,
            'fiction': True,
            'published': ['no_validation_for_type'],
        }, utils.extract_fields_from_dict(source_dict, Book))



# -*- coding: utf-8 -*-
import datetime
from odin import datetimeutil


def date_iso_format(value):
    """
    Serialise a datetime.date to ISO string format.
    """
    assert isinstance(value, datetime.date)
    return value.isoformat()


class DatetimeIsoFormat(object):
    """
    Serialise a datetime.time or datetime.datetime to ISO string format.
    """
    def __init__(self, default_timezone=datetimeutil.local):
        self.default_timezone = default_timezone

    def __call__(self, value):
        assert isinstance(value, (datetime.time, datetime.datetime))
        if value.tzinfo is None:
            value = value.replace(tzinfo=self.default_timezone)
        return value.isoformat()

datetime_iso_format = DatetimeIsoFormat()
TimeIsoFormat = DatetimeIsoFormat
time_iso_format = TimeIsoFormat()


class DatetimeEcmaFormat(object):
    """
    Serialize a datetime object into the ECMA defined format.
    """
    input_type = datetime.datetime

    def __init__(self, assume_local=True, default_timezone=datetimeutil.local):
        self.default_timezone = datetimeutil.local if assume_local else default_timezone

    def __call__(self, value):
        assert isinstance(value, self.input_type)
        return datetimeutil.to_ecma_datetime_string(value, self.default_timezone)

datetime_ecma_format = DatetimeEcmaFormat()



import json
import unittest
import six


class TestCase(unittest.TestCase):
    def assertJSONEqual(self, raw, expected_data, msg=None):
        try:
            data = json.loads(raw)
        except ValueError:
            self.fail("First argument is not valid JSON: %r" % raw)
        if isinstance(expected_data, six.string_types):
            try:
                expected_data = json.loads(expected_data)
            except ValueError:
                self.fail("Second argument is not valid JSON: %r" % expected_data)
        self.assertEqual(data, expected_data, msg=msg)


# -*- coding: utf-8 -*-
import csv
from odin.resources import create_resource_from_dict

CONTENT_TYPE = 'text/csv'


class ResourceReader(csv.DictReader):
    def __init__(self, f, resource, *args, **kwargs):
        self.resource = resource
        csv.DictReader.__init__(self, f, *args, **kwargs)

    # Python 2
    def next(self):
        return create_resource_from_dict(csv.DictReader.next(self), self.resource, copy_dict=False)

    # Python 3
    def __next__(self):
        return create_resource_from_dict(csv.DictReader.__next__(self), self.resource, copy_dict=False)



# -*- coding: utf-8 -*-
import unittest
from . import resources


class TestResourceBase(unittest.TestCase):
    def test_virtual_fields(self):
        target = resources.FromResource()

        self.assertEqual(['constant_field'], [f.name for f in target._meta.virtual_fields])

    def test_virtual_field_inheritance(self):
        target = resources.InheritedResource()

        self.assertEqual(['calculated_field', 'constant_field'], [f.name for f in target._meta.virtual_fields])

    def test_fields(self):
        target = resources.FromResource()

        self.assertEqual(['title', 'count', 'child', 'children', 'excluded1', 'from_field1', 'from_field2',
                          'from_field3', 'from_field4', 'same_but_different', 'from_field_c1', 'from_field_c2',
                          'from_field_c3', 'from_field_c4', 'not_auto_c5', 'comma_separated_string'],
                         [f.name for f in target._meta.fields])

    def test_field_inheritance(self):
        target = resources.InheritedResource()

        self.assertEqual(['name', 'title', 'count', 'child', 'children', 'excluded1', 'from_field1', 'from_field2',
                          'from_field3', 'from_field4', 'same_but_different', 'from_field_c1', 'from_field_c2',
                          'from_field_c3', 'from_field_c4', 'not_auto_c5', 'comma_separated_string'],
                         [f.name for f in target._meta.fields])

    def test_field_multi_inheritance(self):
        target = resources.MultiInheritedResource()

        self.assertEqual(['name', 'title', 'count', 'child', 'children', 'excluded1', 'from_field1', 'from_field2',
                          'from_field3', 'from_field4', 'same_but_different', 'from_field_c1', 'from_field_c2',
                          'from_field_c3', 'from_field_c4', 'not_auto_c5', 'comma_separated_string'],
                         [f.name for f in target._meta.fields])



import unittest
import odin
from odin import traversal


class Level3(odin.Resource):
    class Meta:
        namespace = 'odin.traversal'

    name = odin.StringField()


class Level2(odin.Resource):
    class Meta:
        namespace = 'odin.traversal'

    name = odin.StringField()
    level3s = odin.ListOf(Level3)


class Level1(odin.Resource):
    class Meta:
        namespace = 'odin.traversal'

    name = odin.StringField()
    level2 = odin.DictAs(Level2)
    level2s = odin.DictOf(Level2)


TEST_STRUCTURE = Level1(
    name='a',
    level2=Level2(name='b', level3s=[]),
    level2s=dict(
        a=Level2(name='c', level3s=[]),
        b=Level2(name='d', level3s=[Level3(name='e'), Level3(name='f')]),
        c=Level2(name='g', level3s=[Level3(name='h')]),
    )
)

TEST_LIST_STRUCTURE = [
    Level1(
        name='a',
        level2=Level2(name='b', level3s=[]),
        level2s=dict(
            a=Level2(name='c', level3s=[]),
            b=Level2(name='d', level3s=[Level3(name='e'), Level3(name='f')]),
            c=Level2(name='g', level3s=[Level3(name='h')]),
        )
    ),
    Level1(
        name='i',
        level2=Level2(name='j', level3s=[]),
        level2s=dict(
            a=Level2(name='k', level3s=[]),
            b=Level2(name='l', level3s=[Level3(name='m'), Level3(name='n')]),
            c=Level2(name='o', level3s=[Level3(name='p')]),
        )
    )
]


class TestResourceTraversalIterator(traversal.ResourceTraversalIterator):
    def __init__(self, resource):
        super(TestResourceTraversalIterator, self).__init__(resource)
        self.events = []

    def on_pre_enter(self):
        self.events.append("on_pre_enter: %s" % self.path)

    def on_enter(self):
        self.events.append("on_enter: %s" % self.path)

    def on_exit(self):
        self.events.append("on_exit: %s" % self.path)


class TraversalTestCase(unittest.TestCase):
    def test_structure(self):
        TEST_STRUCTURE.full_clean()

        resource_iter = TestResourceTraversalIterator(TEST_STRUCTURE)
        resources = ["%s %s %s" % (r, r.name, resource_iter.depth) for r in resource_iter]

        self.assertListEqual([
            'on_enter: ',
                'on_enter: level2',
                'on_exit: level2',
                'on_enter: level2s[a]',
                'on_exit: level2s[a]',
                'on_enter: level2s[b]',
                    'on_enter: level2s[b].level3s[0]',
                    'on_exit: level2s[b].level3s[0]',
                    'on_enter: level2s[b].level3s[1]',
                    'on_exit: level2s[b].level3s[1]',
                'on_exit: level2s[b]',
                'on_enter: level2s[c]',
                'on_enter: level2s[c].level3s[0]',
                'on_exit: level2s[c].level3s[0]',
                'on_exit: level2s[c]',
            'on_exit: ',
        ], resource_iter.events)

        self.assertListEqual([
            'odin.traversal.Level1 resource a 0',
            'odin.traversal.Level2 resource b 1',
            'odin.traversal.Level2 resource c 1',
            'odin.traversal.Level2 resource d 1',
            'odin.traversal.Level3 resource e 2',
            'odin.traversal.Level3 resource f 2',
            'odin.traversal.Level2 resource g 1',
            'odin.traversal.Level3 resource h 2',
        ], resources)

    def test_list_structure(self):
        TEST_STRUCTURE.full_clean()

        resource_iter = TestResourceTraversalIterator(TEST_LIST_STRUCTURE)
        resources = ["%s %s %s" % (r, r.name, resource_iter.depth) for r in resource_iter]

        self.assertListEqual([
            'on_enter: ',
                'on_enter: level2',
                'on_exit: level2',
                'on_enter: level2s[a]',
                'on_exit: level2s[a]',
                'on_enter: level2s[b]',
                    'on_enter: level2s[b].level3s[0]',
                    'on_exit: level2s[b].level3s[0]',
                    'on_enter: level2s[b].level3s[1]',
                    'on_exit: level2s[b].level3s[1]',
                'on_exit: level2s[b]',
                'on_enter: level2s[c]',
                'on_enter: level2s[c].level3s[0]',
                'on_exit: level2s[c].level3s[0]',
                'on_exit: level2s[c]',
            'on_exit: ',
            'on_enter: ',
                'on_enter: level2',
                'on_exit: level2',
                'on_enter: level2s[a]',
                'on_exit: level2s[a]',
                'on_enter: level2s[b]',
                    'on_enter: level2s[b].level3s[0]',
                    'on_exit: level2s[b].level3s[0]',
                    'on_enter: level2s[b].level3s[1]',
                    'on_exit: level2s[b].level3s[1]',
                'on_exit: level2s[b]',
                'on_enter: level2s[c]',
                'on_enter: level2s[c].level3s[0]',
                'on_exit: level2s[c].level3s[0]',
                'on_exit: level2s[c]',
            'on_exit: ',
        ], resource_iter.events)

        self.assertListEqual([
            'odin.traversal.Level1 resource a 0',
            'odin.traversal.Level2 resource b 1',
            'odin.traversal.Level2 resource c 1',
            'odin.traversal.Level2 resource d 1',
            'odin.traversal.Level3 resource e 2',
            'odin.traversal.Level3 resource f 2',
            'odin.traversal.Level2 resource g 1',
            'odin.traversal.Level3 resource h 2',
            'odin.traversal.Level1 resource i 0',
            'odin.traversal.Level2 resource j 1',
            'odin.traversal.Level2 resource k 1',
            'odin.traversal.Level2 resource l 1',
            'odin.traversal.Level3 resource m 2',
            'odin.traversal.Level3 resource n 2',
            'odin.traversal.Level2 resource o 1',
            'odin.traversal.Level3 resource p 2',
        ], resources)

class TraversalPathTestCase(unittest.TestCase):
    def test_parse(self):
        actual = traversal.TraversalPath.parse('level2')
        self.assertEqual(traversal.TraversalPath((None, 'level2'),), actual)

        actual = traversal.TraversalPath.parse('level2.name')
        self.assertEqual(traversal.TraversalPath((None, 'level2'), (None, 'name')), actual)

        actual = traversal.TraversalPath.parse('level2s[b].level3s[1].name')
        self.assertEqual(traversal.TraversalPath(('b', 'level2s'), ('1', 'level3s'), (None, 'name')), actual)

    def test_add(self):
        actual = traversal.TraversalPath.parse('level2') + 'name'
        self.assertEqual(traversal.TraversalPath.parse('level2.name'), actual)

        actual = traversal.TraversalPath.parse('level2s[b]') + traversal.TraversalPath.parse('level3s[1].name')
        self.assertEqual(traversal.TraversalPath.parse('level2s[b].level3s[1].name'), actual)

    def test_valid_path(self):
        self.assertEqual('a', traversal.TraversalPath.parse('name').get_value(TEST_STRUCTURE))
        self.assertEqual('b', traversal.TraversalPath.parse('level2.name').get_value(TEST_STRUCTURE))

        r = traversal.TraversalPath.parse('level2s[b].level3s[1]').get_value(TEST_STRUCTURE)
        self.assertIsInstance(r, Level3)
        self.assertEqual('f', r.name)

    def test_invalid_path(self):
        path = traversal.TraversalPath.parse('level2s[b].level3s[4]')
        self.assertRaises(IndexError, path.get_value, TEST_STRUCTURE)

        path = traversal.TraversalPath.parse('level2s[b].level3s_sd[1]')
        self.assertRaises(KeyError, path.get_value, TEST_STRUCTURE)



# -*- coding: utf-8 -*-
from __future__ import absolute_import
import unittest

from odin.resources import ResourceOptions, build_object_graph
import odin
from odin.exceptions import ValidationError
from .resources import Book, Library, Subscriber


class Author(odin.Resource):
    name = odin.StringField()
    country = odin.StringField(null=True)

    def clean(self):
        if self.name == "Bruce" and self.country.startswith("Australia"):
            raise ValidationError("No no no no")

    def clean_country(self, value):
        if value not in ["Australia", "New Zealand"]:
            raise ValidationError("What are ya?")
        return "%s!" % value

    def extra_attrs(self, attrs):
        self.extras = attrs


# Resources for abstract namespace test.
class ResourceA(odin.Resource):
    class Meta:
        abstract = True
        namespace = "example"


class ResourceB(ResourceA):
    class Meta:
        abstract = True


class ResourceC(ResourceB):
    pass


class ResourceD(ResourceC):
    pass


class ResourceTestCase(unittest.TestCase):
    def test_constructor_kwargs_only(self):
        r = Author(name="Foo")
        self.assertEqual("Foo", r.name)

    def test_constructor_kwargs_only_with_unknown_field(self):
        with self.assertRaises(TypeError):
            Author(name="Foo", age=42)

    def test_constructor_args_only(self):
        r = Author("Foo")
        self.assertEqual("Foo", r.name)
        self.assertEqual(None, r.country)

    def test_constructor_args_excess(self):
        with self.assertRaises(TypeError) as ex:
            Author("Foo", "Australia", 42)
        self.assertEqual("This resource takes 2 positional arguments but 3 where given.", str(ex.exception))

    def test_constructor_args_and_kwargs(self):
        r = Author("Foo", country="Australia")
        self.assertEqual("Foo", r.name)
        self.assertEqual("Australia", r.country)

    def test_constructor_args_and_kwargs_overrides(self):
        r = Author("Foo", name="Bar", country="Australia")
        self.assertEqual("Foo", r.name)
        self.assertEqual("Australia", r.country)

    def test_simple_methods(self):
        r = Author()
        self.assertEqual("<Author: tests.test_resources.Author resource>", repr(r))
        self.assertEqual("tests.test_resources.Author resource", str(r))

    def test_clean_fields_1(self):
        r = Author(name="Foo")
        r.country = "Australia"

        r.clean_fields()

        self.assertEqual("Foo", r.name)
        self.assertEqual("Australia!", r.country)

    def test_clean_fields_2(self):
        r = Author(name="Foo")

        r.clean_fields()

        self.assertEqual("Foo", r.name)
        self.assertEqual(None, r.country)

    def test_clean_fields_3(self):
        r = Author(name="Foo", country="England")

        try:
            r.clean_fields()
        except ValidationError as ve:
            self.assertEqual(["What are ya?"], ve.message_dict['country'])
        else:
            raise AssertionError("ValidationError not raised.")

    def test_clean_fields_4(self):
        r = Author()

        try:
            r.clean_fields()
        except ValidationError as ve:
            self.assertEqual(["This field cannot be null."], ve.message_dict['name'])
        else:
            raise AssertionError("ValidationError not raised.")

    def test_full_clean(self):
        r = Author(name="Bruce", country="Australia")

        try:
            r.full_clean()
        except ValidationError as ve:
            self.assertEqual(["No no no no"], ve.message_dict['__all__'])
        else:
            raise AssertionError("ValidationError not raised.")

    def test_full_clean_exclude(self):
        r = Author(name="Bruce", country="England")

        r.full_clean(exclude=('country',))

        self.assertEqual("Bruce", r.name)
        self.assertEqual("England", r.country)

    # Fix for #11
    def test_multiple_abstract_namespaces(self):
        self.assertEqual('example.ResourceC', ResourceC._meta.resource_name)

    def test_parents_1(self):
        self.assertListEqual([], ResourceA._meta.parents)
        self.assertListEqual([ResourceA], ResourceB._meta.parents)
        self.assertListEqual([ResourceA, ResourceB], ResourceC._meta.parents)
        self.assertListEqual([ResourceA, ResourceB, ResourceC], ResourceD._meta.parents)


class MetaOptionsTestCase(unittest.TestCase):
    def test_invalid_options(self):
        class Meta:
            random_val = 10

        class NewResource(object):
            pass

        target = ResourceOptions(Meta)

        with self.assertRaises(TypeError):
            target.contribute_to_class(NewResource, 'etc')


class ConstructionMethodsTestCase(unittest.TestCase):
    def test_build_object_graph_empty_dict_no_clean(self):
        book = build_object_graph({}, Book, full_clean=False)

        self.assertEqual(dict(
            title=None,
            num_pages=None,
            rrp=20.4,
            fiction=None,
            genre=None,
            published=None,
            authors=None,
            publisher=None
        ), book.to_dict())

    def test_build_object_graph_empty_dict(self):
        with self.assertRaises(ValidationError) as ctx:
            build_object_graph({}, Book)

        self.assertEqual(dict(
            title=['This field cannot be null.'],
            num_pages=['This field cannot be null.'],
            fiction=['This field cannot be null.'],
            genre=['This field cannot be null.'],
            published=['This field cannot be null.'],
            authors=['List cannot contain null entries.'],
        ), ctx.exception.error_messages)

    def test_build_object_graph_from_list(self):
        books = build_object_graph([dict(
            title="Book1"
        ), dict(
            title="Book2"
        )], Book, full_clean=False)

        self.assertEqual([dict(
            title="Book1",
            num_pages=None,
            rrp=20.4,
            fiction=None,
            genre=None,
            published=None,
            authors=None,
            publisher=None
        ), dict(
            title="Book2",
            num_pages=None,
            rrp=20.4,
            fiction=None,
            genre=None,
            published=None,
            authors=None,
            publisher=None
        )], [book.to_dict() for book in books])

    def test_build_nested_objects(self):
        subscribers = [
            {'name': 'John Smith', 'address': 'Oak Lane 1234'},
            {'name': 'Johnny Smith', 'address': 'Oak Lane 1235'}]

        library = {
            'name': 'John Smith Library',
            'subscribers': subscribers
        }

        expected = sorted(build_object_graph(subscribers, resource=Subscriber, full_clean=False), key=lambda s: s.name)
        actual = sorted(build_object_graph(library, resource=Library, full_clean=False).subscribers, key=lambda s: s.name)

        self.assertEqual(actual, expected)

    def test_build_nested_objects_with_polymorphism(self):
        books = [{'title': "Book1", 'num_pages': 1, 'rrp': 20.4, 'fiction': True, 'genre': 'sci-fi', 'published': [],
                  'authors': [], 'publisher': None, '$': 'tests.resources.Book'},
                 {'title': "Book2", 'num_pages': 1, 'rrp': 20.4, 'fiction': True, 'genre': 'sci-fi', 'published': [],
                  'authors': [], 'publisher': None, '$': 'tests.resources.Book'}]

        library = {
            'name': 'John Smith Library',
            'books': books
        }

        expected = sorted(build_object_graph(books, full_clean=False), key=lambda s: s.title)
        actual = sorted(build_object_graph(library, resource=Library, full_clean=False).books, key=lambda s: s.title)

        self.assertEqual(actual, expected)



# -*- coding: utf-8 -*-
# This file is largely verbatim from the Django project, the wheel works well, no need to re-invent it.
#
# A note: to use validators from the Django project install the baldr package. Baldr is an integration between Odin and
# the Django framework, the integration includes support for handling the Django version of the ValidationError
# exception within Odin.
import re
import six
from odin import exceptions

EMPTY_VALUES = (None, '', [], (), {})


class RegexValidator(object):
    regex = r''
    message = 'Enter a valid value.'
    code = 'invalid'

    def __init__(self, regex=None, message=None, code=None):
        if regex is not None:
            self.regex = regex
        if message is not None:
            self.message = message
        if code is not None:
            self.code = code

        # Compile the regex if it was not passed pre-compiled.
        if isinstance(self.regex, six.string_types):
            self.regex = re.compile(self.regex)

    def __call__(self, value):
        """
        Validates that the input matches the regular expression.
        """
        if not self.regex.search(value):
            raise exceptions.ValidationError(self.message, code=self.code)


class URLValidator(RegexValidator):
    regex = re.compile(
        r'^(?:http|ftp)s?://'  # http:// or https://
        r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|'  # domain...
        r'localhost|'  # localhost...
        r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}|'  # ...or ipv4
        r'\[?[A-F0-9]*:[A-F0-9:]+\]?)'  # ...or ipv6
        r'(?::\d+)?'  # optional port
        r'(?:/?|[/?]\S+)$', re.IGNORECASE)
    message = "Enter a valid URL value."

validate_url = URLValidator()


class BaseValidator(object):
    compare = lambda self, a, b: a is not b
    clean = lambda self, x: x
    message = 'Ensure this value is %(limit_value)s (it is %(show_value)s).'
    code = 'limit_value'
    description = 'Ensure that a value is %(limit_value)s.'

    def __init__(self, limit_value):
        self.limit_value = limit_value

    def __call__(self, value):
        cleaned = self.clean(value)
        params = {'limit_value': self.limit_value, 'show_value': cleaned}
        if self.compare(cleaned, self.limit_value):
            raise exceptions.ValidationError(self.message % params, code=self.code, params=params)

    def __str__(self):
        return self.description % {'limit_value': self.limit_value}


class MaxValueValidator(BaseValidator):
    compare = lambda self, a, b: a > b
    message = 'Ensure this value is less than or equal to %(limit_value)s.'
    code = 'max_value'
    description = 'Ensure value is less than or equal to %(limit_value)s.'


class MinValueValidator(BaseValidator):
    compare = lambda self, a, b: a < b
    message = 'Ensure this value is greater than or equal to %(limit_value)s.'
    code = 'min_value'
    description = 'Ensure value is greater than or equal to %(limit_value)s.'


class LengthValidator(BaseValidator):
    compare = lambda self, a, b: a != b
    clean = lambda self, x: len(x)
    message = 'Ensure this value has exactly %(limit_value)d characters (it has %(show_value)d).'
    code = 'length'
    description = 'Ensure value has exactly %(limit_value)d characters.'


class MaxLengthValidator(LengthValidator):
    compare = lambda self, a, b: a > b
    message = 'Ensure this value has at most %(limit_value)d characters (it has %(show_value)d).'
    code = 'max_length'
    description = 'Ensure value has at most %(limit_value)d characters.'


class MinLengthValidator(LengthValidator):
    compare = lambda self, a, b: a < b
    message = 'Ensure this value has at least %(limit_value)d characters (it has %(show_value)d).'
    code = 'min_length'
    description = 'Ensure value has at least %(limit_value)d characters.'


def simple_validator(assertion=None, message='The supplied value is invalid', code='invalid'):
    """
    Create a simple validator.

    :param assertion: An Validation exception will be raised if this check returns a none True value.
    :param message: Message to raised in Validation exception if validation fails.
    :param code: Code to included in Validation exception. This can be used to customise the message at the resource
        level.

    Usage::

        >>> none_validator = simple_validator(lambda x: x is not None, message="This value cannot be none")

    This can also be used as a decorator::

        @create_validator(message="This value cannot be none")
        def none_validator(v):
            return v is not None
    """
    def inner(func):
        def wrapper(value):
            params = {'show_value': value}
            if not func(value):
                raise exceptions.ValidationError(message % params, code=code, params=params)
        return wrapper

    if assertion:
        return inner(assertion)
    else:
        return inner



# -*- coding: utf-8 -*-
import six


def _split_atom(atom):
    field, _, key = atom.rstrip(']').partition('[')
    return key or None, field


class TraversalPath(object):
    """
    A path through a resource structure.
    """
    @classmethod
    def parse(cls, path):
        if isinstance(path, six.string_types):
            return cls(*[_split_atom(a) for a in path.split('.')])

    def __init__(self, *path):
        self._path = path

    def __repr__(self):
        return "<TraversalPath: %s>" % self

    def __str__(self):
        return '.'.join("%s" % f if k is None else "%s[%s]" % (f, k) for k, f in self._path)

    def __hash__(self):
        return hash(self._path)

    def __eq__(self, other):
        if isinstance(other, TraversalPath):
            return hash(self) == hash(other)
        return False

    def __add__(self, other):
        if isinstance(other, TraversalPath):
            return TraversalPath(*(self._path + other._path))

        # Assume appending a field
        if isinstance(other, six.string_types):
            return TraversalPath(*(self._path + tuple([(None, other)])))

        raise TypeError("Cannot add '%s' to a path." % other)

    def __iter__(self):
        return iter(self._path)

    def get_value(self, root_resource):
        """
        Get a value from a resource structure.
        """
        value = root_resource
        for key, attr in self:
            field = value._meta.field_map[attr]
            value = field.value_from_object(value)
            if key is not None:
                key = field.key_to_python(key)
                value = value[key]
        return value


class ResourceTraversalIterator(object):
    """
    Iterator for traversing (walking) a resource structure, including traversing composite fields to fully navigate a
    resource tree.

    This class has hooks that can be used by subclasses to customise the behaviour of the class:

     - *on_enter* - Called after entering a new resource.
     - *on_exit* - Called after exiting a resource.

    """
    def __init__(self, resource):
        if isinstance(resource, (list, tuple)):
            # Stack of resource iterators (starts initially with entries from the list)
            self._resource_iters = [iter([(i, r) for i, r in enumerate(resource)])]
        else:
            # Stack of resource iterators (starts initially with single entry of the root resource)
            self._resource_iters = [iter([(None, resource)])]
        # Stack of composite fields, found on each resource, each composite field is interrogated for resources.
        self._field_iters = []
        # The "path" to the current resource.
        self._path = [(None, None)]
        self._resource_stack = [None]

    def __iter__(self):
        return self

    def __next__(self):
        if self._resource_iters:
            if self._field_iters:
                # Check if the last entry in the field stack has any unprocessed fields.
                if self._field_iters[-1]:
                    # Select the very last field in the field stack.
                    field = self._field_iters[-1][0]
                    # Request a list of resources along with keys from the composite field.
                    self._resource_iters.append(field.item_iter_from_object(self.current_resource))
                    # Update the path
                    self._path.append((None, field.name))
                    self._resource_stack.append(None)
                    # Remove the field from the list (and remove this field entry if it has been emptied)
                    self._field_iters[-1].pop(0)
                else:
                    self._field_iters.pop()

            if self.current_resource:
                if hasattr(self, 'on_exit'):
                    self.on_exit()

            try:
                key, next_resource = next(self._resource_iters[-1])
            except StopIteration:
                # End of the current list of resources pop this list off and get the next list.
                self._path.pop()
                self._resource_iters.pop()
                self._resource_stack.pop()

                return next(self)
            else:
                # If we have a key (ie DictOf, ListOf composite fields) update the path key field.
                if key is not None:
                    _, field = self._path[-1]
                    self._path[-1] = (key, field)

                # Get list of any composite fields for this resource (this is a cached field).
                self._field_iters.append(list(next_resource._meta.composite_fields))

                # self.current_resource = next_resource
                self._resource_stack[-1] = next_resource

                if hasattr(self, 'on_enter'):
                    self.on_enter()
                return next_resource
        else:
            raise StopIteration()

    # Python 2.x compatibility
    next = __next__

    @property
    def path(self):
        """
        Path to the current resource node in the tree structure.

        This path can be used to later traverse the tree structure to find get to the specified resource.
        """
        # The path is offset by one as the path includes the root to simplify next method.
        return TraversalPath(*self._path[1:])

    @property
    def depth(self):
        """
        Depth of the current resource in the tree structure.
        """
        return len(self._path) - 1

    @property
    def current_resource(self):
        if self._resource_stack:
            return self._resource_stack[-1]



# -*- coding: utf-8 -*-
import odin
from odin.fields.virtual import CalculatedField
from odin.mapping.helpers import sum_fields


class Author(odin.Resource):
    name = odin.StringField()

    class Meta:
        name_space = None


class Publisher(odin.Resource):
    name = odin.StringField()

    class Meta:
        name_space = None


class LibraryBook(odin.Resource):
    class Meta:
        abstract = True
        name_space = "library"


class Book(LibraryBook):
    title = odin.StringField()
    num_pages = odin.IntegerField()
    rrp = odin.FloatField(default=20.4, use_default_if_not_provided=True)
    fiction = odin.BooleanField(is_attribute=True)
    genre = odin.StringField(choices=(
        ('sci-fi', 'Science Fiction'),
        ('fantasy', 'Fantasy'),
        ('biography', 'Biography'),
        ('others', 'Others'),
    ))
    published = odin.TypedArrayField(odin.DateTimeField())
    authors = odin.ArrayOf(Author, use_container=True)
    publisher = odin.DictAs(Publisher, null=True)

    def __eq__(self, other):
        if other:
            return vars(self) == vars(other)

        return False


class Subscriber(odin.Resource):
    name = odin.StringField()
    address = odin.StringField()

    def __eq__(self, other):
        if other:
            return self.name == other.name and self.address == other.address

        return None


class Library(odin.Resource):
    name = odin.StringField()
    books = odin.ArrayOf(LibraryBook)
    subscribers = odin.ArrayOf(Subscriber, null=True)
    book_count = CalculatedField(lambda o: len(o.books))

    class Meta:
        name_space = None


class OldBook(LibraryBook):
    name = odin.StringField()
    num_pages = odin.IntegerField()
    price = odin.FloatField()
    genre = odin.StringField(choices=(
        ('sci-fi', 'Science Fiction'),
        ('fantasy', 'Fantasy'),
        ('biography', 'Biography'),
        ('others', 'Others'),
    ))
    published = odin.DateTimeField()
    author = odin.ObjectAs(Author)
    publisher = odin.ObjectAs(Publisher)


class OldBookToBookMapping(odin.Mapping):
    from_obj = OldBook
    to_obj = Book

    exclude_fields = ('',)

    mappings = (
        ('name', None, 'title'),
    )


class ChildResource(odin.Resource):
    name = odin.StringField()


class FromResource(odin.Resource):
    # Auto matched
    title = odin.StringField()
    count = odin.StringField()
    child = odin.ObjectAs(ChildResource)
    children = odin.ArrayOf(ChildResource)
    # Excluded
    excluded1 = odin.FloatField()
    # Mappings
    from_field1 = odin.StringField()
    from_field2 = odin.StringField()
    from_field3 = odin.IntegerField()
    from_field4 = odin.IntegerField()
    same_but_different = odin.StringField()
    # Custom mappings
    from_field_c1 = odin.StringField()
    from_field_c2 = odin.StringField()
    from_field_c3 = odin.StringField()
    from_field_c4 = odin.StringField()
    not_auto_c5 = odin.StringField()
    comma_separated_string = odin.StringField()
    # Virtual fields
    constant_field = odin.ConstantField(value=10)


class InheritedResource(FromResource):
    # Additional fields
    name = odin.StringField()
    # Additional virtual fields
    calculated_field = odin.CalculatedField(lambda obj: 11)


class MultiInheritedResource(InheritedResource, FromResource):
    pass


class ToResource(odin.Resource):
    # Auto matched
    title = odin.StringField()
    count = odin.IntegerField()
    child = odin.ObjectAs(ChildResource)
    children = odin.ArrayOf(ChildResource)
    # Excluded
    excluded1 = odin.FloatField()
    # Mappings
    to_field1 = odin.StringField()
    to_field2 = odin.IntegerField()
    to_field3 = odin.IntegerField()
    same_but_different = odin.StringField()
    # Custom mappings
    to_field_c1 = odin.StringField()
    to_field_c2 = odin.StringField()
    to_field_c3 = odin.StringField()
    not_auto_c5 = odin.StringField()
    array_string = odin.TypedArrayField(odin.StringField())
    assigned_field = odin.StringField()


class FromToMapping(odin.Mapping):
    from_obj = FromResource
    to_obj = ToResource

    exclude_fields = ('excluded1',)

    mappings = (
        ('from_field1', None, 'to_field1'),
        ('from_field2', int, 'to_field2'),
        (('from_field3', 'from_field4'), sum_fields, 'to_field3'),
        ('from_field1', None, 'same_but_different'),
    )

    @odin.map_field(from_field=('from_field_c1', 'from_field_c2', 'from_field_c3'), to_field='to_field_c1')
    def multi_to_one(self, *values):
        return '-'.join(values)

    @odin.map_field(from_field='from_field_c4', to_field=('to_field_c2', 'to_field_c3'))
    def one_to_multi(self, value):
        return value.split('-', 1)

    @odin.map_field
    def not_auto_c5(self, value):
        return value.upper()

    @odin.map_list_field(to_field='array_string')
    def comma_separated_string(self, value):
        return value.split(',')

    @odin.assign_field
    def assigned_field(self):
        return 'Foo'



# -*- coding: utf-8 -*-
from pint.unit import DimensionalityError
import six
from odin import exceptions
from odin.contrib.pint.units import registry
from odin.fields import Field
from odin.validators import EMPTY_VALUES

__all__ = ('FloatField',)


class PintField(Field):
    def __init__(self, units, **kwargs):
        super(PintField, self).__init__(**kwargs)

        # Ensure we have valid units
        if units is None:
            raise ValueError("Units cannot be None")
        if isinstance(units, six.string_types):
            units = registry[units]
        # if not units_ in units.registry:
        #    raise ValueError("Units object is not a member of `odin.units.registry`. Any custom units must be "
        #                     "registered with Odin's unit registry.")
        self.units = units

    def to_quantity(self, value):
        if not isinstance(value, registry.Quantity):
            # Split out units from underlying magnitudes
            if isinstance(value, (tuple, list)) and len(value) == 2:
                raw_value, units = value
            else:
                raw_value, units = value, self.units
            # Extract the units
            if isinstance(units, six.string_types):
                units = registry[units]
            # Convert the magnitude and apply units
            value = self.to_magnitude(raw_value) * units

        try:
            return value.to(self.units)
        except DimensionalityError as de:
            raise exceptions.ValidationError(de)

    def to_magnitude(self, value):
        raise NotImplementedError()


class FloatField(PintField):
    default_error_messages = {
        'invalid': "'%s' value must be a float.",
    }
    data_type_name = "Float"

    def to_magnitude(self, value):
        try:
            return float(value)
        except ValueError:
            msg = self.error_messages['invalid'] % value
            raise exceptions.ValidationError(msg)

    def to_python(self, value):
        if value in EMPTY_VALUES:
            return None
        return self.to_quantity(value)



from setuptools import setup, find_packages

try:
    long_description = open("README.rst").read()
except IOError:
    long_description = ""

setup(
    name='odin',
    version='0.6.0.1',
    url='https://github.com/timsavage/odin',
    license='LICENSE',
    author='Tim Savage',
    author_email='tim@savage.company',
    description='Object Data Mapping for Python',
    long_description=long_description,
    packages=find_packages(),
    install_requires=['six'],
    extras_require={
        # Documentation generation
        'doc_gen': ["jinja2>=2.7"],
        # Pint integration
        'pint': ["pint"],
    },

    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)
